@@ -1,4 +1,4 @@
-# Copyright 2012-2013 GRNET S.A. All rights reserved.
+# Copyright 2012-2014 GRNET S.A. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or
 # without modification, are permitted provided that the following
@@ -32,33 +32,79 @@
 # or implied, of GRNET S.A.
 
 from logging import getLogger
-from astakosclient import AstakosClient as OriginalAstakosClient
-from astakosclient import AstakosClientException, parse_endpoints
+import inspect
+import ssl
 
-from kamaki.clients import Client, ClientError, RequestManager, recvlog
+from astakosclient import AstakosClientException, parse_endpoints
+import astakosclient
+
+from kamaki.clients import (
+    Client, ClientError, KamakiSSLError, RequestManager, recvlog)
+
+from kamaki.clients.utils import https
+
+
+log = getLogger(__name__)
+
+
+class AstakosClientError(ClientError, AstakosClientException):
+    """Join AstakosClientException as ClientError in one class"""
+
+    def __init__(self, message='Astakos Client Error', details='', status=0):
+        super(ClientError, self).__init__(message, details, status)
+
+
+def _astakos_error(foo):
+    def wrap(self, *args, **kwargs):
+        try:
+            return foo(self, *args, **kwargs)
+        except AstakosClientException as sace:
+            if isinstance(getattr(sace, 'errobject', None), ssl.SSLError):
+                raise KamakiSSLError('SSL Connection error (%s)' % sace)
+            raise AstakosClientError(
+                getattr(sace, 'message', '%s' % sace),
+                details=sace.details, status=sace.status)
+    return wrap
+
+
+#  Patch AstakosClient to support SSLAuthentication
+astakosclient.utils.PooledHTTPConnection = https.PooledHTTPConnection
+astakosclient.utils.HTTPSConnection = https.HTTPSClientAuthConnection
+OriginalAstakosClient = astakosclient.AstakosClient
 
 
 class AstakosClient(OriginalAstakosClient):
-    """Wrap Original AstakosClient to ensure bw compatibility and ease of use
-
-    Note that this is an ancached class, so each call produces at least one
-    new http request
-    """
+    """Wrap Original AstakosClient to ensure compatibility in kamaki clients"""
 
+    @_astakos_error
     def __init__(self, *args, **kwargs):
         if args:
             args = list(args)
             url = args.pop(0)
             token = args.pop(0) if args else kwargs.pop('token', None)
             args = tuple([token, url] + args)
-        elif 'base_url' in kwargs:
-            kwargs['auth_url'] = kwargs.get('auth_url', kwargs['base_url'])
+        else:
+            kwargs.setdefault(
+                'auth_url', kwargs.get('endpoint_url', kwargs['base_url']))
+
+        # If no CA certificates are set, get the defaults from kamaki.defaults
+        if https.HTTPSClientAuthConnection.ca_file is None:
+            try:
+                from kamaki import defaults
+                https.HTTPSClientAuthConnection.ca_file = getattr(
+                    defaults, 'CACERTS_DEFAULT_PATH', None)
+            except ImportError as ie:
+                log.debug('ImportError while loading default certs: %s' % ie)
+
         super(AstakosClient, self).__init__(*args, **kwargs)
 
     def get_service_endpoints(self, service_type, version=None):
         services = parse_endpoints(
             self.get_endpoints(), ep_type=service_type, ep_version_id=version)
-        return services[0]['endpoints'][0] if services else []
+        return services[0]['endpoints'][0] if services else {}
+
+    def get_endpoint_url(self, service_type, version=None):
+        return self.get_service_endpoints(service_type, version)['publicURL']
 
     @property
     def user_info(self):
@@ -68,13 +114,10 @@ class AstakosClient(OriginalAstakosClient):
         return self.user_info[term]
 
 
-def _astakos_error(foo):
-    def wrap(self, *args, **kwargs):
-        try:
-            return foo(self, *args, **kwargs)
-        except AstakosClientException as sace:
-            self._raise_for_status(sace)
-    return wrap
+#  Wrap AstakosClient public methods to raise AstakosClientError
+for m in inspect.getmembers(AstakosClient):
+    if hasattr(m[1], '__call__') and not ('%s' % m[0]).startswith('_'):
+        setattr(AstakosClient, m[0], _astakos_error(m[1]))
 
 
 class LoggedAstakosClient(AstakosClient):
@@ -87,6 +130,10 @@ class LoggedAstakosClient(AstakosClient):
     LOG_TOKEN = False
     LOG_DATA = False
 
+    def __init__(self, *args, **kwargs):
+        kwargs.setdefault('logger', log)
+        super(LoggedAstakosClient, self).__init__(*args, **kwargs)
+
     def _dump_response(self, request, status, message, data):
         recvlog.info('\n%d %s' % (status, message))
         recvlog.info('data size: %s' % len(data))
@@ -119,17 +166,18 @@ class LoggedAstakosClient(AstakosClient):
                         message=log_response['message'],
                         data=log_response.get('data', ''))
         except Exception:
-            pass
+            recvlog.debug('Kamaki failed to log an AstakosClient call')
         finally:
             return r
 
 
 class CachedAstakosClient(Client):
     """Synnefo Astakos cached client wraper"""
+    service_type = 'identity'
 
     @_astakos_error
-    def __init__(self, base_url, token=None):
-        super(CachedAstakosClient, self).__init__(base_url, token)
+    def __init__(self, endpoint_url, token=None):
+        super(CachedAstakosClient, self).__init__(endpoint_url, token)
         self._astakos = dict()
         self._uuids = dict()
         self._cache = dict()
@@ -162,8 +210,7 @@ class CachedAstakosClient(Client):
         :param token: (str) custom token to authenticate
         """
         token = self._resolve_token(token)
-        astakos = LoggedAstakosClient(
-            self.base_url, token, logger=getLogger('astakosclient'))
+        astakos = LoggedAstakosClient(self.endpoint_url, token, logger=log)
         astakos.LOG_TOKEN = getattr(self, 'LOG_TOKEN', False)
         astakos.LOG_DATA = getattr(self, 'LOG_DATA', False)
         r = astakos.authenticate()
@@ -206,7 +253,7 @@ class CachedAstakosClient(Client):
 
         :returns: (dict) {name:..., type:..., endpoints:[...]}
 
-        :raises ClientError: (600) if service_type not in service catalog
+        :raises AstakosClientError: if service_type not in service catalog
         """
         services = self.get_services(token)
         for service in services:
@@ -215,8 +262,8 @@ class CachedAstakosClient(Client):
                     return service
             except KeyError:
                 self.log.warning('Misformated service %s' % service)
-        raise ClientError(
-            'Service type "%s" not in service catalog' % service_type, 600)
+        raise AstakosClientError(
+            'Service type "%s" not in service catalog' % service_type)
 
     def get_service_endpoints(self, service_type, version=None, token=None):
         """
@@ -226,9 +273,8 @@ class CachedAstakosClient(Client):
 
         :returns: (dict) {SNF:uiURL, adminURL, internalURL, publicURL, ...}
 
-        :raises ClientError: (600) if service_type not in service catalog
-
-        :raises ClientError: (601) if #matching endpoints != 1
+        :raises AstakosClientError: if service_type not in service catalog, or
+            if #matching endpoints != 1
         """
         service = self.get_service_details(service_type, token)
         matches = []
@@ -237,13 +283,16 @@ class CachedAstakosClient(Client):
                     endpoint['versionId'].lower() == version.lower()):
                 matches.append(endpoint)
         if len(matches) != 1:
-            raise ClientError(
+            raise AstakosClientError(
                 '%s endpoints match type %s %s' % (
                     len(matches), service_type,
-                    ('and versionId %s' % version) if version else ''),
-                601)
+                    ('and versionId %s' % version) if version else ''))
         return matches[0]
 
+    def get_endpoint_url(self, service_type, version=None, token=None):
+        r = self.get_service_endpoints(service_type, version, token)
+        return r['publicURL']
+
     def list_users(self):
         """list cached users information"""
         if not self._cache:
@@ -270,7 +319,7 @@ class CachedAstakosClient(Client):
         return self.user_info(token).get(key, None)
 
     def post_user_catalogs(self, uuids=None, displaynames=None, token=None):
-        """POST base_url/user_catalogs
+        """POST endpoint_url/user_catalogs
 
         :param uuids: (list or tuple) user uuids
 
