@@ -1,4 +1,4 @@
-# Copyright 2012-2013 GRNET S.A. All rights reserved.
+# Copyright 2012-2014 GRNET S.A. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or
 # without modification, are permitted provided that the following
@@ -36,6 +36,7 @@ from kamaki.clients.utils import path4url
 
 
 class PithosRestClient(StorageClient):
+    service_type = 'object-store'
 
     def account_head(
             self,
@@ -59,6 +60,8 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
+        self.response_headers = ['Last-Modified', ]
+        self.response_header_prefices = ['X-Account-', ]
 
         self._assert_account()
         path = path4url(self.account)
@@ -68,7 +71,11 @@ class PithosRestClient(StorageClient):
         self.set_header('If-Unmodified-Since', if_unmodified_since)
 
         success = kwargs.pop('success', 204)
-        return self.head(path, *args, success=success, **kwargs)
+        r = self.head(path, *args, success=success, **kwargs)
+        self._unquote_header_keys(
+            r.headers,
+            ('x-account-group-', 'x-account-policy-', 'x-account-meta-'))
+        return r
 
     def account_get(
             self,
@@ -76,6 +83,7 @@ class PithosRestClient(StorageClient):
             marker=None,
             format='json',
             show_only_shared=False,
+            public=False,
             until=None,
             if_modified_since=None,
             if_unmodified_since=None,
@@ -108,13 +116,15 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_account()
+        self.response_headers = ['Last-Modified', ]
+        self.response_header_prefices = ['X-Account-', ]
 
         self.set_param('limit', limit, iff=limit)
         self.set_param('marker', marker, iff=marker)
         self.set_param('format', format, iff=format)
         self.set_param('shared', iff=show_only_shared)
+        self.set_param('public', iff=public)
         self.set_param('until', until, iff=until)
 
         self.set_header('If-Modified-Since', if_modified_since)
@@ -154,10 +164,11 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_account()
 
         self.set_param('update', '', iff=update)
+        self.request_header_prefices_to_quote = [
+            'x-account-meta-', 'x-account-group-']
 
         if groups:
             for group, usernames in groups.items():
@@ -172,6 +183,8 @@ class PithosRestClient(StorageClient):
                 self.set_header('X-Account-Meta-' + metaname, metaval)
         self.set_header('X-Account-Policy-Quota', quota)
         self.set_header('X-Account-Policy-Versioning', versioning)
+        self._quote_header_keys(
+            self.headers, ('x-account-group-', 'x-account-meta-'))
 
         path = path4url(self.account)
         success = kwargs.pop('success', 202)
@@ -199,8 +212,9 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = ['Last-Modified', ]
+        self.response_header_prefices = ['X-Container-', ]
 
         self.set_param('until', until, iff=until)
 
@@ -209,7 +223,10 @@ class PithosRestClient(StorageClient):
 
         path = path4url(self.account, self.container)
         success = kwargs.pop('success', 204)
-        return self.head(path, *args, success=success, **kwargs)
+        r = self.head(path, *args, success=success, **kwargs)
+        self._unquote_header_keys(
+            r.headers, ('x-container-policy-', 'x-container-meta-'))
+        return r
 
     def container_get(
             self,
@@ -221,6 +238,7 @@ class PithosRestClient(StorageClient):
             format='json',
             meta=[],
             show_only_shared=False,
+            public=False,
             until=None,
             if_modified_since=None,
             if_unmodified_since=None,
@@ -266,6 +284,8 @@ class PithosRestClient(StorageClient):
         """
 
         self._assert_container()
+        self.response_headers = ['Last-Modified', ]
+        self.response_header_prefices = ['X-Container-', ]
 
         self.set_param('limit', limit, iff=limit)
         self.set_param('marker', marker, iff=marker)
@@ -276,6 +296,7 @@ class PithosRestClient(StorageClient):
             self.set_param('path', path)
         self.set_param('format', format, iff=format)
         self.set_param('shared', iff=show_only_shared)
+        self.set_param('public', iff=public)
         if meta:
             self.set_param('meta',  ','.join(meta))
         self.set_param('until', until, iff=until)
@@ -289,7 +310,7 @@ class PithosRestClient(StorageClient):
 
     def container_put(
             self,
-            quota=None, versioning=None, metadata=None,
+            quota=None, versioning=None, project_id=None, metadata=None,
             *args, **kwargs):
         """ Full Pithos+ PUT at container level
 
@@ -305,12 +326,18 @@ class PithosRestClient(StorageClient):
         :returns: ConnectionResponse
         """
         self._assert_container()
+        self.request_header_prefices_to_quote = ['x-container-meta-', ]
 
         self.set_header('X-Container-Policy-Quota', quota)
         self.set_header('X-Container-Policy-Versioning', versioning)
+        if project_id is not None:
+            self.set_header('X-Container-Policy-Project', project_id)
+
         if metadata:
             for metaname, metaval in metadata.items():
                 self.set_header('X-Container-Meta-' + metaname, metaval)
+        self._quote_header_keys(
+            self.headers, ('x-container-policy-', 'x-container-meta-'))
 
         path = path4url(self.account, self.container)
         success = kwargs.pop('success', (201, 202))
@@ -322,6 +349,7 @@ class PithosRestClient(StorageClient):
             format='json',
             quota=None,
             versioning=None,
+            project_id=None,
             metadata=None,
             content_type=None,
             content_length=None,
@@ -353,18 +381,24 @@ class PithosRestClient(StorageClient):
         :returns: ConnectionResponse
         """
         self._assert_container()
+        self.request_header_prefices_to_quote = ['x-container-meta-', ]
 
         self.set_param('update', '', iff=update)
         self.set_param('format', format, iff=format)
 
         self.set_header('X-Container-Policy-Quota', quota)
         self.set_header('X-Container-Policy-Versioning', versioning)
+        if project_id is not None:
+            self.set_header('X-Container-Policy-Project', project_id)
+
         if metadata:
             for metaname, metaval in metadata.items():
                 self.set_header('X-Container-Meta-' + metaname, metaval)
         self.set_header('Content-Type', content_type)
         self.set_header('Content-Length', content_length)
         self.set_header('Transfer-Encoding', transfer_encoding)
+        self._quote_header_keys(
+            self.headers, ('x-container-policy-', 'x-container-meta-'))
 
         path = path4url(self.account, self.container)
         success = kwargs.pop('success', 202)
@@ -420,8 +454,16 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = [
+            'ETag',
+            'Content-Length',
+            'Content-Type',
+            'Last-Modified',
+            'Content-Encoding',
+            'Content-Disposition',
+        ]
+        self.response_header_prefices = ['X-Object-', ]
 
         self.set_param('version', version, iff=version)
 
@@ -432,7 +474,9 @@ class PithosRestClient(StorageClient):
 
         path = path4url(self.account, self.container, obj)
         success = kwargs.pop('success', 200)
-        return self.head(path, *args, success=success, **kwargs)
+        r = self.head(path, *args, success=success, **kwargs)
+        self._unquote_header_keys(r.headers, 'x-object-meta-')
+        return r
 
     def object_get(
             self, obj,
@@ -476,8 +520,17 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = [
+            'ETag',
+            'Content-Length',
+            'Content-Type',
+            'Last-Modified',
+            'Content-Encoding',
+            'Content-Disposition',
+            'Content-Range',
+        ]
+        self.response_header_prefices = ['X-Object-', ]
 
         self.set_param('format', format, iff=format)
         self.set_param('hashmap', hashmap, iff=hashmap)
@@ -492,7 +545,9 @@ class PithosRestClient(StorageClient):
 
         path = path4url(self.account, self.container, obj)
         success = kwargs.pop('success', 200)
-        return self.get(path, *args, success=success, **kwargs)
+        r = self.get(path, *args, success=success, **kwargs)
+        self._unquote_header_keys(r.headers, ('x-object-meta-'))
+        return r
 
     def object_put(
             self, obj,
@@ -571,8 +626,10 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = ['ETag', 'X-Object-Version', ]
+        self.request_headers_to_quote = ['x-copy-from', 'x-move-from', ]
+        self.request_header_prefices_to_quote = ['x-object-meta-', ]
 
         self.set_param('format', format, iff=format)
         self.set_param('hashmap', hashmap, iff=hashmap)
@@ -605,6 +662,7 @@ class PithosRestClient(StorageClient):
         if metadata:
             for key, val in metadata.items():
                 self.set_header('X-Object-Meta-' + key, val)
+        self._quote_header_keys(self.headers, ('x-object-meta-', ))
 
         path = path4url(self.account, self.container, obj)
         success = kwargs.pop('success', 201)
@@ -668,8 +726,20 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = [
+            'If-Match',
+            'If-None-Match',
+            'Destination',
+            'Destination-Account',
+            'Content-Type',
+            'Content-Encoding',
+            'Content-Disposition',
+            'X-Source-Version',
+        ]
+        self.response_header_prefices = ['X-Object-', ]
+        self.request_header_prefices_to_quote = [
+            'x-object-meta-', 'Destination']
 
         self.set_param('format', format, iff=format)
         self.set_param('ignore_content_type', iff=ignore_content_type)
@@ -695,6 +765,7 @@ class PithosRestClient(StorageClient):
         if metadata:
             for key, val in metadata.items():
                 self.set_header('X-Object-Meta-' + key, val)
+        self._unquote_header_keys(self.headers, 'x-object-meta-')
 
         path = path4url(self.account, self.container, obj)
         success = kwargs.pop('success', 201)
@@ -756,8 +827,20 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = [
+            'If-Match',
+            'If-None-Match',
+            'Destination',
+            'Destination-Account',
+            'Content-Type',
+            'Content-Encoding',
+            'Content-Disposition',
+            'X-Source-Version',
+        ]
+        self.response_header_prefices = ['X-Object-', ]
+        self.request_header_prefices_to_quote = [
+            'x-object-meta-', 'Destination']
 
         self.set_param('format', format, iff=format)
         self.set_param('ignore_content_type', iff=ignore_content_type)
@@ -777,6 +860,7 @@ class PithosRestClient(StorageClient):
         if metadata:
             for key, val in metadata.items():
                 self.set_header('X-Object-Meta-' + key, val)
+        self._unquote_header_keys(self.headers, 'x-object-meta-')
 
         path = path4url(self.account, self.container, object)
         success = kwargs.pop('success', 201)
@@ -855,8 +939,9 @@ class PithosRestClient(StorageClient):
 
         :returns: ConnectionResponse
         """
-
         self._assert_container()
+        self.response_headers = ['ETag', 'X-Object-Version']
+        self.request_header_prefices_to_quote = ['x-object-meta-', ]
 
         self.set_param('format', format, iff=format)
         self.set_param('update', '', iff=update)
@@ -882,6 +967,7 @@ class PithosRestClient(StorageClient):
         self.set_header('X-Object-Public', public, public is not None)
         for key, val in metadata.items():
             self.set_header('X-Object-Meta-' + key, val)
+        self._quote_header_keys(self.headers, ('x-object-meta-', ))
 
         path = path4url(self.account, self.container, obj)
         success = kwargs.pop('success', (202, 204))
