@@ -1,8 +1,8 @@
 
-__version__ = "0.13.1"
+__version__ = "0.13.5"
 __version_vcs_info__ = {
-    'branch': 'master',
-    'revid': '7b1ab62',
-    'revno': 2477}
+    'branch': 'hotfix-0.13.5',
+    'revid': 'dfe824d',
+    'revno': 2509}
 __version_user_email__ = "saxtouri@admin.grnet.gr"
 __version_user_name__ = "Stavros Sachtouris"
